/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: FIFOSelector.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <iostream>
#include <cstdio>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <fcntl.h>

#include "ServerCommon.h"
#include "FIFOManager.h"
#include "FIFOSelector.h"

using namespace std;
using namespace OpenSOAP;


const std::string TAG_ERR = "*** E ***::";

//static 
const std::string FIFOSelector::LOCAL_IP_ADDR ="127.0.0.1";

FIFOSelector::FIFOSelector(const std::string& ip_addr, int port)
  : ip_addr_(ip_addr)
  , port_(port)
{
}

FIFOSelector::~FIFOSelector()
{

}

int FIFOSelector::connectFIFOManager()
{
  typedef struct sockaddr SockAddr;
  typedef struct sockaddr_in SockAddrIn;

  int sockfd = int();
  SockAddrIn serv_addr;
  
  memset((char*)&serv_addr, 0, sizeof(serv_addr));
  serv_addr.sin_family = PF_INET;
  serv_addr.sin_addr.s_addr = inet_addr(ip_addr_.c_str());
  serv_addr.sin_port = htons(port_);

  //FIFOManagerƂ̒ʐM|[g
  if (0 > (sockfd = socket(PF_INET, SOCK_STREAM, 0))) {
    cerr << TAG_ERR << " open socket failed" << endl;
    return -1;
    //throw std::exception();
    // ONX̐݌vKv 
  }

#ifdef DEBUG
  cerr << "FIFOSelector:: socket open OK!" << endl;
#endif

  if (0 > connect(sockfd, (SockAddr*)&serv_addr, sizeof(serv_addr))) {
    cerr << TAG_ERR << "FIFOSelector:: connect failed" << endl;
    return -1;
    //throw std::exception();
    // ONX̐݌vKv 
  }

#ifdef DEBUG
    cerr << "FIFOSelector:: connect OK!" << endl;
#endif

    return sockfd;
}

//==================================================================

FIFOUpsideSelector::FIFOUpsideSelector(int port)
  : FIFOSelector(FIFOSelector::LOCAL_IP_ADDR, port)
{

}

/*
FIFOUpsideSelector::FIFOUpsideSelector(const std::string& ip_addr, int port)
  : FIFOSelector(ip_addr, port)
{

}
*/

FIFOUpsideSelector::~FIFOUpsideSelector()
{
}

//int FIFOUpsideSelector::open(FIFODescriptor& fifo_d)
int FIFOUpsideSelector::open(FIFODescriptor& fifo_d, const int wait_second)
{
  // timeout l̃`FbN
  if (FIFOManager::NO_WAIT != wait_second &&
      FIFOManager::WAIT_NOLIMIT != wait_second &&
      0 > wait_second) {
    cerr << TAG_ERR << "FIFOUpsideSelector::open: invalid timeout second" << endl;
    return -1;
  }

  int sockfd = connectFIFOManager();
  if (0 > sockfd) {
    return sockfd;
  }

  char sendBuf[BUFSIZ];

  sprintf(sendBuf, "%s %d", OPEN_UPSIDE_FIFO, wait_second);

#ifdef DEBUG
  cerr << "FIFOSelector:: write ready" << endl;
#endif

  //write(sockfd, sendBuf, strlen(sendBuf));
  write(sockfd, sendBuf);

#ifdef DEBUG
  cerr << "FIFOSelector:: write done." << endl;
#endif

#if 1
  std::string recvData;
  int recvLen = read(sockfd, recvData);

#ifdef DEBUG
  cerr << "FIFOUpsideSelector::open: read:[" << recvData << "] len=(" << recvLen << ")" << endl;
#endif
  
#else
  char recvBuf[BUFSIZ];
  memset(recvBuf, 0x00, sizeof(recvBuf));
  ssize_t recvLen = 0;
  std::string recvData;
  while (0 != (recvLen = read(sockfd, recvBuf, sizeof(recvBuf)))) {
    
#ifdef DEBUG
  cerr << "FIFOUpsideSelector::open: read:[" << recvBuf << "] len=(" << recvLen << ")" << endl;
#endif

    if (0 > recvLen) {
      cerr << TAG_ERR << "FIFOUpsideSelector::open: read failed" << endl;
      return -1;
    }

    recvData += recvBuf;
    memset(recvBuf, 0x00, sizeof(recvBuf));
  }
#endif

  ::close(sockfd);

  //Mf[^̉

  const std::string delims(" \t");
  //std::string recvData(recvBuf);
  std::string::size_type begIdx, endIdx;
  begIdx = recvData.find_first_not_of(delims);
  if (begIdx == std::string::npos) {
    cerr << "FIFOSelector:: all blank" << endl;
    return -1;
  }
  endIdx = recvData.find_first_of(delims, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = recvData.length();
  }
  std::string result = recvData.substr(begIdx, (endIdx-begIdx));
  cerr << "FIFOSelector:: result=[" << result << "]" << endl;
  
  if ("FAILURE" == result) {
    cerr << "FIFOSelector:: result failed" << endl;
    return 1;
    
  } else if ("SUCCESS" == result) {
    cerr << "FIFOSelector:: result OK!!" << endl;
    
  } else {
    cerr << "FIFOSelector:: invalid data recived" << endl;
    return -1;
  }
  
  // ID
  begIdx = recvData.find_first_not_of(delims, endIdx);
  if (begIdx == std::string::npos) {
    cerr << "FIFOSelector:: not recv id" << endl;
    return -1;
  }
  endIdx = recvData.find_first_of(delims, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = recvData.length();
  }
  std::string id = recvData.substr(begIdx, (endIdx-begIdx));
  cerr << "FIFOSelector:: id=[" << id << "]" << endl;
  
  // UP
  begIdx = recvData.find_first_not_of(delims, endIdx);
  if (begIdx == std::string::npos) {
    cerr << "FIFOSelector:: not recv id" << endl;
    return -1;
  }
  endIdx = recvData.find_first_of(delims, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = recvData.length();
  }
  std::string up_fifo = recvData.substr(begIdx, (endIdx-begIdx));
  cerr << "FIFOSelector:: up_fifo=[" << up_fifo << "]" << endl;
  
  // DOWN
  begIdx = recvData.find_first_not_of(delims, endIdx);
  if (begIdx == std::string::npos) {
    cerr << "not recv id" << endl;
    return -1;
  }
  endIdx = recvData.find_first_of(delims, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = recvData.length();
  }
  std::string down_fifo = recvData.substr(begIdx, (endIdx-begIdx));
  cerr << "down_fifo=[" << down_fifo << "]" << endl;


  //FIFOManager瓾FIFOt@Copen

  int r_fd = ::open(up_fifo.c_str(), O_RDONLY);  
  if (0 > r_fd) {
    cerr << TAG_ERR << " FIFO open failed : [" << up_fifo << "]" << endl;
    return -1;
  }

  int w_fd = ::open(down_fifo.c_str(), O_WRONLY);
  if (0 > w_fd) {
    cerr << TAG_ERR << " FIFO open failed : [" << down_fifo << "]" << endl;
    return -1;
  }

  fifo_d.setId(id);
  fifo_d.setReadFIFO(r_fd);
  fifo_d.setWriteFIFO(w_fd);

  // sendFIFOManager("OPEN DONE");
  // FIFOManager->setBUSY

  return 0;
}

int FIFOUpsideSelector::close(const FIFODescriptor& fifo_d)
{
  //FIFOclose
  //std::close(fifo_d.getReadFIFO());
  ::close(fifo_d.getWriteFIFO());
  ::close(fifo_d.getReadFIFO());

  int sockfd = connectFIFOManager();
  if (0 > sockfd) {
    return sockfd;
  }

  char sendBuf[BUFSIZ];

  sprintf(sendBuf, "%s %s", CLOSE_UPSIDE_FIFO, fifo_d.getId().c_str());

#ifdef DEBUG
  cerr << "FIFOSelector:: write ready" << endl;
#endif

  //write(sockfd, sendBuf, strlen(sendBuf));
  write(sockfd, sendBuf);
  
#ifdef DEBUG
  cerr << "FIFOSelector:: write done." << endl;
#endif

#if 1
  std::string recvData;
  int recvLen = read(sockfd, recvData);

#ifdef DEBUG
  cerr << "FIFOUpsideSelector::close: read:[" << recvData << "] len=(" << recvLen << ")" << endl;
#endif
  
#else
  char recvBuf[BUFSIZ];
  memset(recvBuf, 0x00, sizeof(recvBuf));

  ssize_t recvLen = 0;
  std::string recvData;
  while (0 != (recvLen = read(sockfd, recvBuf, sizeof(recvBuf)))) {
    
#ifdef DEBUG
  cerr << "FIFOUpsideSelector::close: read:[" << recvBuf << "] len=(" << recvLen << ")" << endl;
#endif

    if (0 > recvLen) {
      cerr << TAG_ERR << "FIFOUpsideSelector::close: read failed" << endl;
      return -1;
    }

    recvData += recvBuf;
    memset(recvBuf, 0x00, sizeof(recvBuf));
  }
#endif

  ::close(sockfd);

  //Mf[^̉

  const std::string delims(" \t");
  //std::string recvData(recvBuf);
  std::string::size_type begIdx, endIdx;
  begIdx = recvData.find_first_not_of(delims);
  if (begIdx == std::string::npos) {
    cerr << "all blank" << endl;
    return -1;
  }
  endIdx = recvData.find_first_of(delims, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = recvData.length();
  }
  std::string result = recvData.substr(begIdx, (endIdx-begIdx));
  cerr << "result=[" << result << "]" << endl;
  
  if ("FAILURE" == result) {
    cerr << "result failed" << endl;
    return 1;
    
  } else if ("SUCCESS" == result) {
    cerr << "result OK!!" << endl;
    
  } else {
    cerr << "invalid data recived" << endl;
    return -1;
  }

  return 0;
}

//==================================================================

FIFODownsideSelector::FIFODownsideSelector(int port)
  : FIFOSelector(FIFOSelector::LOCAL_IP_ADDR, port)
{

}

/*
FIFODownsideSelector::FIFODownsideSelector(const std::string& ip_addr, int port)
  : FIFOSelector(ip_addr, port)
{
}
*/

FIFODownsideSelector::~FIFODownsideSelector()
{
}

//int FIFODownsideSelector::open(FIFODescriptor& fifo_d)
int FIFODownsideSelector::open(FIFODescriptor& fifo_d, const int wait_second)
{
  // timeout l̃`FbN
  if (FIFOManager::NO_WAIT != wait_second &&
      FIFOManager::WAIT_NOLIMIT != wait_second &&
      0 > wait_second) {
    cerr << TAG_ERR << "FIFODownsideSelector::open: invalid timeout second" << endl;
    return -1;
  }

  int sockfd = connectFIFOManager();
  if (0 > sockfd) {
    //return sockfd;
    return -101;
  }

  char sendBuf[BUFSIZ];

  sprintf(sendBuf, "%s %d", OPEN_DOWNSIDE_FIFO, wait_second);

#ifdef DEBUG
  cerr << "FIFOSelector:: write ready" << endl;
#endif

  //FIFOManagerփR}hM
  //write(sockfd, sendBuf, strlen(sendBuf));
  write(sockfd, sendBuf);

#ifdef DEBUG
  cerr << "FIFOSelector:: write done." << endl;
#endif

#if 1
  std::string recvData;
  int recvLen = read(sockfd, recvData);

#ifdef DEBUG
  cerr << "FIFODownsideSelector::open: read:[" << recvData << "] len=(" << recvLen << ")" << endl;
#endif
  
#else
  char recvBuf[BUFSIZ];
  memset(recvBuf, 0x00, sizeof(recvBuf));
  //FIFOManager猋ʎM
  ssize_t recvLen = 0;
  std::string recvData;
  while (0 != (recvLen = read(sockfd, recvBuf, sizeof(recvBuf)))) {
    
#ifdef DEBUG
  cerr << "FIFODownsideSelector::open: read:[" << recvBuf << "] len=(" << recvLen << ")" << endl;
#endif

    if (0 > recvLen) {
      cerr << TAG_ERR << "FIFODownsideSelector::open: read failed" << endl;
      return -1;
    }

    recvData += recvBuf;
    memset(recvBuf, 0x00, sizeof(recvBuf));
  }
#endif

  ::close(sockfd);

  //Mf[^̉
  const std::string delims(" \t");
  //std::string recvData(recvBuf);
  std::string::size_type begIdx, endIdx;
  begIdx = recvData.find_first_not_of(delims);
  if (begIdx == std::string::npos) {
    cerr << "all blank" << endl;
    return -102;
  }
  endIdx = recvData.find_first_of(delims, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = recvData.length();
  }
  std::string result = recvData.substr(begIdx, (endIdx-begIdx));
  cerr << "result=[" << result << "]" << endl;
  
  if ("FAILURE" == result) {
    cerr << "result failed" << endl;
    return 1;
    
  } else if ("SUCCESS" == result) {
    cerr << "result OK!!" << endl;
    
  } else {
    cerr << "invalid data recived" << endl;
    return -102;
  }
  
  // ID
  begIdx = recvData.find_first_not_of(delims, endIdx);
  if (begIdx == std::string::npos) {
    cerr << "not recv id" << endl;
    return -102;
  }
  endIdx = recvData.find_first_of(delims, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = recvData.length();
  }
  std::string id = recvData.substr(begIdx, (endIdx-begIdx));
  cerr << "id=[" << id << "]" << endl;
  
  // UP
  begIdx = recvData.find_first_not_of(delims, endIdx);
  if (begIdx == std::string::npos) {
    cerr << "not recv id" << endl;
    return -102;
  }
  endIdx = recvData.find_first_of(delims, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = recvData.length();
  }
  std::string up_fifo = recvData.substr(begIdx, (endIdx-begIdx));
  cerr << "up_fifo=[" << up_fifo << "]" << endl;
  
  // DOWN
  begIdx = recvData.find_first_not_of(delims, endIdx);
  if (begIdx == std::string::npos) {
    cerr << "not recv id" << endl;
    return -102;
  }
  endIdx = recvData.find_first_of(delims, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = recvData.length();
  }
  std::string down_fifo = recvData.substr(begIdx, (endIdx-begIdx));
  cerr << "down_fifo=[" << down_fifo << "]" << endl;


  //FIFOManager瓾FIFOt@Copen

  int w_fd = ::open(up_fifo.c_str(), O_WRONLY);
  if (0 > w_fd) {
    cerr << TAG_ERR << " FIFO open failed : [" << up_fifo << "]" << endl;
    return -101;
  }

  int r_fd = ::open(down_fifo.c_str(), O_RDONLY);
  if (0 > r_fd) {
    cerr << TAG_ERR << " FIFO open failed : [" << down_fifo << "]" << endl;
    return -101;
  }

  fifo_d.setId(id);
  fifo_d.setReadFIFO(r_fd);
  fifo_d.setWriteFIFO(w_fd);

  return 0;
}

int FIFODownsideSelector::close(const FIFODescriptor& fifo_d)
{
  //FIFOclose
  ::close(fifo_d.getWriteFIFO());
  ::close(fifo_d.getReadFIFO());

  int sockfd = connectFIFOManager();
  if (0 > sockfd) {
    return sockfd;
  }
  
  char sendBuf[BUFSIZ];
  
  sprintf(sendBuf, "%s %s", CLOSE_DOWNSIDE_FIFO, fifo_d.getId().c_str());

#ifdef DEBUG
  cerr << "FIFOSelector:: write ready" << endl;
#endif

  //write(sockfd, sendBuf, strlen(sendBuf));
  write(sockfd, sendBuf);
  
#ifdef DEBUG
  cerr << "FIFOSelector:: write done." << endl;
#endif

#if 1
  std::string recvData;
  int recvLen = read(sockfd, recvData);

#ifdef DEBUG
  cerr << "FIFODownsideSelector::close: read:[" << recvData << "] len=(" << recvLen << ")" << endl;
#endif
  
#else
  char recvBuf[BUFSIZ];
  memset(recvBuf, 0x00, sizeof(recvBuf));
  ssize_t recvLen = 0;
  std::string recvData;
  while (0 != (recvLen = read(sockfd, recvBuf, sizeof(recvBuf)))) {
    
#ifdef DEBUG
  cerr << "FIFODownsideSelector::close: read:[" << recvBuf << "] len=(" << recvLen << ")" << endl;
#endif

    if (0 > recvLen) {
      cerr << TAG_ERR << "FIFODownsideSelector::close: read failed" << endl;
      return -1;
    }

    recvData += recvBuf;
    memset(recvBuf, 0x00, sizeof(recvBuf));
  }
#endif

  ::close(sockfd);

  //Mf[^̉

  const std::string delims(" \t");
  //std::string recvData(recvBuf);
  std::string::size_type begIdx, endIdx;
  begIdx = recvData.find_first_not_of(delims);
  if (begIdx == std::string::npos) {
    cerr << "all blank" << endl;
    return -1;
  }
  endIdx = recvData.find_first_of(delims, begIdx);
  if (endIdx == std::string::npos) {
    endIdx = recvData.length();
  }
  std::string result = recvData.substr(begIdx, (endIdx-begIdx));
  cerr << "result=[" << result << "]" << endl;
  
  if ("FAILURE" == result) {
    cerr << "result failed" << endl;
    return 1;
    
  } else if ("SUCCESS" == result) {
    cerr << "result OK!!" << endl;
    
  } else {
    cerr << "invalid data recived" << endl;
    return -1;
  }

  return 0;
}


// End of FIFOSelector.cpp
