/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: QueueManager_main.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <iostream>
#if defined(WIN32)
#include <windows.h>
#include <io.h>
#else
#include <unistd.h>
#endif

#include <string>
#include <fstream>

#if !defined(NO_LOGGER)
#include "Logger.h"
#endif
#include "ServerCommon.h"

#include "QueueManager.h"

using namespace std;
using namespace OpenSOAP;

int 
main(int argc, char* argv[])
{
  // if argument exists, it behaves as Queue Manager of Forwarder
  //bool isFwdQueue = true;
  bool isFwdQueue = false;

#if defined(WIN32)
	string logFile = OPENSOAP_TMP_PATH + "log/queueManager_req";
	if (argc > 2) {
	    if (atoi(argv[2]) != 0) {
			isFwdQueue = true;
			logFile = OPENSOAP_TMP_PATH + "log/queueManager_fwd";
	    }
	}
#else
	if (argc > 1) {
	    if (atoi(argv[1]) != 0) {
		    isFwdQueue = true;
	    }
	}
/*
  if (argc == 1 || (argc > 1 && atoi(argv[1]) == 0)) {
    isFwdQueue = false;
  }
*/
#endif

#if defined(NO_LOGGER)
	if (argc >1) {
		logFile = argv[1];
	}
	FILE* outfp = fopen(logFile.c_str(), "w");
	if (!outfp) {
		return 1;
	}
	if (argc <=1) {
		fprintf(outfp, "invalid initial argument. argc=(%d)\n", argc);
		fprintf(outfp, "exit.\n");
		fclose(outfp);
		return EXIT_FAILURE;
	}
	int old;
	int olderr;
	//backup stdout
	old = _dup(1);
	//stdout -> outfp
	if (-1 == _dup2(_fileno(outfp), 1)) {
		fprintf(outfp, "_dup2 to stdout failed.\n");
		fprintf(outfp, "exit.\n");
		fclose(outfp);
		return EXIT_FAILURE;
	}
	//backup stderr
	olderr = _dup(2);
	//stderr -> outfp
	if (-1 == _dup2(_fileno(outfp), 2)) {
		fprintf(outfp, "_dup2 to stderr failed.\n");
		fprintf(outfp, "exit.\n");
		fclose(outfp);
		return EXIT_FAILURE;
	}

#else
  // set log level  
  Logger::InitOpt opt;
  opt.outputLevel = LOG_DEBUG;
  Logger::init(opt);
  //write start message
  if (isFwdQueue) {
    Logger::printLog(NULL, " QueueManger_main (fwd) start", LOG_INFO,
		     0x00100003L, "exe");
  }
  else {
    Logger::printLog(NULL, " QueueManger_main (req) start", LOG_INFO,
		     0x00100003L, "exe");
  }
#endif

#if !defined(WIN32)
  // make it to daemon & write pid file
#if HAVE_DAEMON
  daemon(0, 1);
#endif
  
  pid_t pid = getpid();
  
  
  ofstream pidFile;
  if (isFwdQueue) {
    // Queue Manager for Forward
    cout << "Forward Queue Manager" << endl;
    pidFile.open(FWDQUEUE_PID_FILE.c_str());
    if (!pidFile) {
      cerr << "Error: pid file [" 
           << FWDQUEUE_PID_FILE << "] cannot open!" << endl;
      exit(1);
    }
    pidFile << pid;
    pidFile.close();
  }
  else {
    // Queue Manager for Request
    cout << "Response Queue Manager" << endl;
    pidFile.open(REQQUEUE_PID_FILE.c_str());
    if (!pidFile) {
      cerr << "Error: pid file [" 
           << REQQUEUE_PID_FILE << "] cannot open!" << endl;
      exit(1);
    }
  }
  pidFile << pid;
  pidFile.close();
#endif

  // create instance of QueueManager and run
  ChannelManager* queueManagerPtr = new QueueManager(isFwdQueue);
  queueManagerPtr->run();
  delete queueManagerPtr;

#if defined(NO_LOGGER)
	fclose(outfp);
	_dup2(old, 1);
	_dup2(olderr, 2);
#else
  //write stop message
  if (isFwdQueue) {
    Logger::printLog(NULL, " QueueManger_main (fwd) stop", LOG_INFO,
		     0x00100003L, "exe");
  }
  else {
    Logger::printLog(NULL, " QueueManger_main (req) stop", LOG_INFO,
		     0x00100003L, "exe");
  }
  Logger::term();
#endif

  return 0;
}

